@@ -15,20 +15,17 @@
 
 
 volcano <-
-  function(matestobj, threshold=c(0.001,0.05,0.05, 0.05),
-           method=c("unadj","unadj", "unadj", "unadj"), title="Volcano Plot",
+  function(matestobj, threshold=c(0.001,0.05),
+           method=c("unadj","unadj"), title="Volcano Plot",
            highlight.flag=TRUE, onScreen=TRUE)
 {
   if(class(matestobj)[1] != "matest")
     stop("The first input variable is not an object of class matest.")
 
-  #must have F1
-  if( !("F1" %in% names(matestobj)) )
-    stop("No F1 test result in input object. Cannot do volcano plot")
-  #must have F3
-  if( !("F3" %in% names(matestobj)) )
-    stop("No F3 test result in input object. Cannot do volcano plot")
-       
+  # must have F1
+  if( !("F1" %in% names(matestobj))  )
+    stop("No Fs test result in input object. Cannot do volcano plot")
+
   if(class(matestobj)[2] == "ttest")
     # this is a T-test result
     result <- volcano.ttest(matestobj, threshold, method, title,
@@ -44,84 +41,52 @@ volcano <-
 # function to do f-test volcano plot
 ######################################################################
 volcano.ftest <- function(matestobj, threshold, method, title,
-                          highlight.flag)
-{
-  anovaobj <- matestobj$obsAnova
-  
-  if(length(method) == 1)
-    method <- rep(method, 4)
+                          highlight.flag){
+  anovaobj = matestobj$obsAnova
+  probeid = matestobj$probeid
+
+  if(length(method) == 1) method <- rep(method, 2)
   
   # local variables
   th.f1 <- threshold[1]
-  th.f2 <- threshold[2]
-  th.f3 <- threshold[3]
-  th.fs <- threshold[4]
+  th.fs <- threshold[2]
+  #th.fss <- threshold[3]
   ngenes <- length(anovaobj$G)
-  
+
   # get P values for all F tests according to method
   # for F1
   p1 <- getPval.volcano(matestobj, method, 1)
-  idx1 <- p1 < th.f1
-  # for F2
-  p2 <- getPval.volcano(matestobj, method, 2)
-  idx2 <- p2 < th.f2
-  # for F3
-  p3 <- getPval.volcano(matestobj, method, 3)
-  idx3 <- p3 < th.f3
+  idx1 <- p1 < th.f1 ; 
   # for Fs
-  ps <- getPval.volcano(matestobj, method, 4)
-  idxs <- ps < th.fs
-
+  ps <- getPval.volcano(matestobj, method, 2)
+  idxs <- ps < th.fs ; 
+  # for Fss
+  #pss <- getPval.volcano(matestobj, method, 3)
+  #idxss <- pss < th.fss; ; 
   
   # calculate the x axis value
-  # the x-axis value should be the numerator of the test, e.g.,
+  # the x-axis value should be the numerator of the test, e.g.
   # (Lb)'*pinv((L*pinv(X'X)*L')*(Lb)
-  xvalue <- calVolcanoXval(matestobj)
-  # use F3$Fobs as the x-axis value
-  #xvalue <- matestobj$F3$Fobs
 
+  xvalue <- calVolcanoXval(matestobj)
   # y-axis value
   yvalue <- -log10(p1)
-  
+
   # plot the figure
-  xlabel <- "" #F3 test F value"
-  plot(xvalue, yvalue, xlab=xlabel, ylab="-log10(Ptab)",
+  plot(xvalue, yvalue, xlab="Log2(FoldChange)", ylab="-log10(Pvalue)",
        col="blue", pch=4, cex=0.5, main=title)
 
   # draw a reference line based on the threshold of f1
   abline(h=-log10(th.f1))
   
-  # replot the significant genes from F2 (if there) in red
-  if("F2" %in% names(matestobj))
-    points(xvalue[idx2], yvalue[idx2], col="red", pch=4, cex=0.5)
-
-  # replot the significant genes from F3 (if there) in green
-  if("F3" %in% names(matestobj))
-    points(xvalue[idx3], yvalue[idx3], col="green", pch=4, cex=0.5)
-  
-  # plot the significant genes from Fs (if there) in orange
+  # replot the significant genes from Fs (if there) in red
   if("Fs" %in% names(matestobj))
-    points(xvalue[idxs], yvalue[idxs], col="orange", pch=4, cex=0.5)
+    points(xvalue[idxs], yvalue[idxs], col="red", pch=4, cex=0.5)
+ 
+  # plot the significant genes from Fss (if there) in orange
+  #if("Fss" %in% names(matestobj))
+  #  points(xvalue[idxss], yvalue[idxss], col="orange", pch=4, cex=0.5)
 
-
-  # draw the reference line based on F3 test (if there)
-#  vshape <- 0
-#  if("F3" %in% names(matestobj)) {
-#    if(sum(idx3) != 0) {
-      # find the location of the reference line
-#      l <- min(abs(xvalue[idx3]))
-#      if(vshape == 1) {
-#        abline(v=l)
-#        abline(v=-l)
-#      }
-#      else {
-#        abline(v=l)
-#      }
-#    }
-#    else
-#      warning("There is no significant genes from F3 test.")
-#  }
-  
   # circle the flaged genes (if any)
   if(highlight.flag) {
     if(!is.null(anovaobj$flag)) {
@@ -129,26 +94,24 @@ volcano.ftest <- function(matestobj, threshold, method, title,
       points(xvalue[idx.flag], yvalue[idx.flag])
     }
   }
-  
+
   # find the significant genes from all three F tests
   result <- NULL
-  result$idx.F1 <- which(idx1)
-  idx <- result$idx.F1
-  if("F2" %in% names(matestobj)) {
-    result$idx.F2 <- which(idx2)
-    idx <- intersect(idx, result$idx.F2)
-  }
-  if("F3" %in% names(matestobj)) {
-    result$idx.F3 <- which(idx3)
-    idx <- intersect(idx, result$idx.F3)
-  }
+  idx.F1 <- which(idx1); idx <- idx.F1
+  names(idx.F1) = probeid[idx.F1]
+  result$idx.F1 =idx.F1
+
   if("Fs" %in% names(matestobj)) {
-    result$idx.Fs <- which(idxs)
-      idx <- intersect(idx, result$idx.Fs)
+    idx.Fs <- which(idxs); idx <- intersect(idx, idx.Fs)
+    names(idx.Fs) = probeid[idx.Fs]
+    result$idx.Fs <- idx.Fs 
   }
-
-  result$idx.all <- idx
-
+  #if("Fss" %in% names(matestobj)) {
+  #  idx.Fss <- which(idxss); idx <- intersect(idx, idx.Fss)
+  #  names(idx.Fss) = probeid[idx.Fss]
+  #  result$idx.Fss <- idx.Fss 
+  #}
+  idx.all = idx; names(idx.all) = probeid[idx.all]; result$idx.all <- idx.all
   result
 }
 
@@ -161,32 +124,29 @@ volcano.ftest <- function(matestobj, threshold, method, title,
 volcano.ttest <- function(matestobj, threshold, method, title,
                           highlight.flag, onScreen)
 {
-  anovaobj <- matestobj$obsAnova
+  anovaobj = matestobj$obsAnova
+  probeid = matestobj$probeid
   
   if(length(method) == 1)
-    method <- rep(method, 4)
+    method <- rep(method, 2)
   
   # local variables
   th.f1 <- threshold[1]
-  th.f2 <- threshold[2]
-  th.f3 <- threshold[3]
-  th.fs <- threshold[4]
+  th.fs <- threshold[2]
+  #th.fss <- threshold[3]
   ngenes <- length(anovaobj$G)
   
   # get P values for all F tests according to method
   # for F1
   p1 <- getPval.volcano(matestobj, method, 1)
   idx1 <- p1 < th.f1
-  # for F2
-  p2 <- getPval.volcano(matestobj, method, 2)
-  idx2 <- p2 < th.f2
-  # for F3
-  p3 <- getPval.volcano(matestobj, method, 3)
-  idx3 <- p3 < th.f3
   # for Fs
-  ps <- getPval.volcano(matestobj, method, 4)
+  ps <- getPval.volcano(matestobj, method, 2)
   idxs <- ps < th.fs
-  
+  # for Fss
+  #pss <- getPval.volcano(matestobj, method, 3)
+  #idxss <- pss < th.fss
+ 
   #############################
   # calculate the x axis value
   #############################
@@ -201,29 +161,21 @@ volcano.ttest <- function(matestobj, threshold, method, title,
 
   # calculate x-axis values
   xvalue.all <- calVolcanoXval(matestobj)
-  #xvalue.all <- matestobj$F3$Fobs
 
   # loop for contrasts
+  levels <- NULL
+  for(i in 1:length(diff.terms)) {
+    # level names
+    levels <- c(levels, anovaobj[[paste(diff.terms[i], ".level", sep="")]])
+  }
   for(icon in 1:nplots) {
-    levels <- NULL
-    for(i in 1:length(diff.terms)) {
-      # level names
-      levels <- c(levels, anovaobj[[paste(diff.terms[i], ".level", sep="")]])
-    }
     # xvalue for this contrast
     xvalue <- xvalue.all[,icon]
     
     ######### make x-axis labels
-    # first term
-#    if(L[1] == 0) xlabel <- NULL
-#    if(L[1] == 1) xlabel <- levels[1]
-#    else if(L[1] == -1) xlabel <- paste("-", level[1]j, sep="")
-#    else xlabel <- paste(L[1], level[1], sep="*")
-    # rest of them
-#    for(i in 2:length(L)) {
-#      xlabel <- paste(xlabel, "+")}
-#    xlabel <- paste(paste(L, levels, sep="*"), collapse="+")
-    xlabel <- "F3 test F values"
+    xlabel = levels[Contrast[icon, ] !=0]
+    xlabel <- paste(paste(xlabel, sep="*"), collapse="-")
+    xlabel <- paste('Log2(FoldChange) of ', xlabel, sep="", collapse=" ")
 
     # figure title
     title <- paste("comparison", icon)
@@ -231,36 +183,28 @@ volcano.ttest <- function(matestobj, threshold, method, title,
     # start to plot
     if(onScreen) {
       # open a window on screen
-      get(getOption("device"))()
- #    if(.Platform$GUI == "AQUA")
- #       quartz()
- #     else
- #       x11()
+      dev.new()
     }
     # y-axis value
     yvalue <- -log10(p1[,icon])
     # plot the figure
-    plot(xvalue, yvalue, xlab=xlabel, ylab="-log10(Ptab)",
+    plot(xvalue, yvalue, xlab=xlabel, ylab="-log10(Pvalue)",
          col="blue", pch=4, cex=0.5, main=title)
   
     # draw a reference line based on the threshold of f1
-    abline(h=-log10(th.f1))
+    abline(h = -log10(th.f1))
     idx1 <- p1[,icon] < th.f1
-    # replot the significant genes from F2 (if there) in red
-    if("F2" %in% names(matestobj)) {
-      idx2 <- p2[,icon] < th.f2
-      points(xvalue[idx2], yvalue[idx2], col="red", pch=4, cex=0.5)
-    }
-    # replot the significant genes from F3 (if there) in green
-    if("F3" %in% names(matestobj)) {
-      idx3 <- p3[,icon] < th.f3
-      points(xvalue[idx3], yvalue[idx3], col="green", pch=4, cex=0.5)
-    }
-    # plot the significant genes from Fs (if there) in orange
+ 
+    # replot the significant genes from Fs (if there) in green
     if("Fs" %in% names(matestobj)) {
       idxs <- ps[,icon] < th.fs
-      points(xvalue[idxs], yvalue[idxs], col="orange", pch=4, cex=0.5)
+      points(xvalue[idxs], yvalue[idxs], col="red", pch=4, cex=0.5)
     }
+    # plot the significant genes from Fss (if there) in orange
+    #if("Fss" %in% names(matestobj)) {
+    #  idxss <- pss[,icon] < th.fss
+    #  points(xvalue[idxss], yvalue[idxss], col="orange", pch=4, cex=0.5)
+    #}
 
     # draw the reference line based on F3 test (if there)
     # note that this is alway vshape
@@ -291,27 +235,27 @@ volcano.ttest <- function(matestobj, threshold, method, title,
     }
   
     # find the significant genes from all three F tests
+
     result.tmp <- NULL
-    result.tmp$idx.F1 <- which(idx1)
-    idx <- result.tmp$idx.F1
-    if("F2" %in% names(matestobj)) {
-      result.tmp$idx.F2 <- which(idx2)
-      idx <- intersect(idx, result.tmp$idx.F2)
-    }
-    if("F3" %in% names(matestobj)) {
-      result.tmp$idx.F3 <- which(idx3)
-      idx <- intersect(idx, result.tmp$idx.F3)
-    }
+    idx.F1 <- which(idx1); idx <- idx.F1
+    names(idx.F1) = probeid[idx.F1]
+    result.tmp$idx.F1 <- idx.F1
+
+    #if("Fss" %in% names(matestobj)) {
+    #  idx.Fss <- which(idxss); idx <- intersect(idx, idx.Fss)
+    # names(idx.Fss) = probeid[idx.Fss]
+    # result.tmp$idx.Fss <- idx.Fss 
+    #}
     if("Fs" %in% names(matestobj)) {
-      result.tmp$idx.Fs <- which(idxs)
-      idx <- intersect(idx, result.tmp$idx.Fs)
+      idx.Fs <- which(idxs); idx <- intersect(idx, idx.Fs)
+      names(idx.Fs) = probeid[idx.Fs]
+      result.tmp$idx.Fs <- idx.Fs 
     }
-    
-    result.tmp$idx.all <- idx
+    idx.all <- idx
+    names(idx.all) = probeid[idx.all]; result.tmp$idx.all <- idx.all
     result.name <- paste("comparison", icon, sep="")
     result[[result.name]] <- result.tmp
   }
-  
   result
 }
 
@@ -323,10 +267,12 @@ volcano.ttest <- function(matestobj, threshold, method, title,
 getPval.volcano <- function(matestobj, method, idx)
 {
   # field name
-  if(idx == 4)
+  if(idx == 2)
     whichF <- "Fs"
-  else
-    whichF <- paste("F", idx, sep="")
+  if(idx == 1)
+    whichF <- "F1"
+  #if(idx == 3)
+  #  whichF <- "Fss"
 
   if( !(whichF %in% names(matestobj)) )
     return(NULL)
@@ -368,7 +314,8 @@ getPval.volcano <- function(matestobj, method, idx)
 calVolcanoXval <- function(matestobj)
 {
   model <- matestobj$model
-  
+  anovaobj = matestobj$obsAnova
+
   # get the estimates from ANOVA object on observed data
   term <- matestobj$term
   parsed.formula <- model$parsed.formula
@@ -379,7 +326,7 @@ calVolcanoXval <- function(matestobj)
   for(i in 1:length(termidx)) {
     tmpterm <- term[i]
     # get the estimates
-    tmpb <- matestobj$obsAnova[[tmpterm]]
+    tmpb <- anovaobj[[tmpterm]]
     b <- cbind(b, tmpb)
   }
 
@@ -400,8 +347,6 @@ calVolcanoXval <- function(matestobj)
     # this is a T-test object
     # The volcano will always be 2-sided
     xval <- apply(L, 1, function(x) x%*%t(b))
-  }
-  
-          
+  }        
   xval
 }
